mod apply_mutations;
mod deferred_system;
pub mod hooks;
mod tick;

use self::{
    apply_mutations::{BevyTemplate, Event},
    deferred_system::DeferredSystemRegistry,
    tick::{tick_dioxus_ui, VirtualDomUnsafe},
};
use bevy::{
    app::{App, Plugin, Update},
    ecs::{bundle::Bundle, component::Component, entity::Entity},
    ui::node_bundles::NodeBundle,
    utils::{HashMap, HashSet},
};
use dioxus::core::{Element, ElementId, Scope};

pub use self::deferred_system::DeferredSystem;
pub use bevy_mod_picking;
pub use dioxus;

pub struct DioxusUiPlugin;

impl Plugin for DioxusUiPlugin {
    fn build(&self, app: &mut App) {
        app.init_resource::<DeferredSystemRegistry>()
            .add_systems(Update, tick_dioxus_ui);
    }
}

#[derive(Bundle)]
pub struct DioxusUiBundle {
    pub dioxus_ui_root: DioxusUiRoot,
    pub node_bundle: NodeBundle,
}

#[derive(Component)]
pub struct DioxusUiRoot {
    virtual_dom: VirtualDomUnsafe,
    hierarchy: HashMap<(Entity, u8), Entity>,
    element_id_to_bevy_ui_entity: HashMap<ElementId, Entity>,
    event_listeners: HashSet<(Event, ElementId)>,
    templates: HashMap<String, BevyTemplate>,
    needs_rebuild: bool,
}

impl DioxusUiRoot {
    pub fn new(root_component: fn(Scope) -> Element) -> Self {
        Self {
            virtual_dom: VirtualDomUnsafe::new(root_component),
            hierarchy: HashMap::new(),
            element_id_to_bevy_ui_entity: HashMap::new(),
            event_listeners: HashSet::new(),
            templates: HashMap::new(),
            needs_rebuild: true,
        }
    }
}
